import Combine
import Foundation
import Martin
import SwiftUI

enum AppFlow {
    case start

    case welcome
    case login
    case registration

    case contactsList
    case chatsList
    case settings

    case chat
}

final class AppState: ObservableObject {
    static let shared = AppState()

    private let xmppService: XMPPService
    private let accountsService: AccountsService
    private let rostersService: RostersService
    private let chatService: ChatsService
    private let messageService: MessageService
    private var conversationService: ConversationService? {
        didSet {
            resubscribeCurrentConversation()
        }
    }

    private var cancellables: Set<AnyCancellable> = []
    private var messagesCancelables: Set<AnyCancellable> = []

    // publishers
    @Published var flow: AppFlow = .start
    @Published private(set) var clients: [XMPPClient] = []
    @Published private(set) var accounts: [Account] = []
    @Published private(set) var rosters: [Roster] = []
    @Published private(set) var chats: [Chat] = []
    @Published private(set) var messages: [Message] = []
    private(set) var currentChat: Chat?

    init() {
        let database = Database()
        xmppService = XMPPService(manager: database)
        accountsService = AccountsService(database._db)
        rostersService = RostersService(database._db, xmppService)
        chatService = ChatsService(database._db)
        messageService = MessageService(database._db)

        doSubscriptions()
    }
}

// MARK: - Cross-subscriptions between services
private extension AppState {
    func doSubscriptions() {
        // subscribe to services properties
        xmppService.$clients
            .sink { [weak self] clients in
                self?.clients = clients
            }
            .store(in: &cancellables)
        accountsService.$accounts
            .sink { [weak self] accounts in
                self?.accounts = accounts
            }
            .store(in: &cancellables)
        rostersService.$rosters
            .sink { [weak self] rosters in
                self?.rosters = rosters
            }
            .store(in: &cancellables)
        chatService.$chats
            .sink { [weak self] chats in
                self?.chats = chats
            }
            .store(in: &cancellables)

        // subscribe xmpp service to accounts service
        accountsService.$accounts
            .sink { [weak self] accounts in
                self?.xmppService.updateClients(for: accounts)
            }
            .store(in: &cancellables)

        // subscribe message service to xmpp service
        xmppService.$clients
            .sink { [weak self] clients in
                self?.messageService.updateSubscriptions(for: clients)
            }
            .store(in: &cancellables)
    }

    func resubscribeCurrentConversation() {
        messagesCancelables = Set<AnyCancellable>()
        if let conversationService {
            messageService.activeChat = conversationService.chat
            conversationService.$messages
                .sink { [weak self] messages in
                    DispatchQueue.main.async {
                        withAnimation {
                            self?.messages = messages
                        }
                    }
                }
                .store(in: &messagesCancelables)
            currentChat = conversationService.chat
        } else {
            messageService.activeChat = nil
            currentChat = nil
        }
    }
}

// MARK: - Actions
extension AppState {
    // Accounts
    func addAccount(jid: String, password: String) {
        accountsService.addAccount(jid: jid, password: password)
    }

    func deleteAccount(jid: String) {
        accountsService.deleteAccount(jid: jid)
    }

    // Rosters
    func addRoster(ownerJID: String, contactJID: String, name: String? = nil, groups: [String] = [], completion: @escaping (Result<Void, Error>) -> Void) {
        rostersService.addRoster(ownerJID: ownerJID, contactJID: contactJID, name: name, groups: groups, completion: completion)
    }

    func deleteRoster(ownerJID: String, contactJID: String, completion: @escaping (Result<Void, Error>) -> Void) {
        rostersService.deleteRoster(ownerJID: ownerJID, contactJID: contactJID, completion: completion)
    }

    func markRosterAsLocallyDeleted(ownerJID: String, contactJID: String) {
        rostersService.markRosterAsLocallyDeleted(ownerJID: ownerJID, contactJID: contactJID)
    }

    // Chats
    func startChat(_ chat: Chat) {
        let client = xmppService.getClient(for: chat.account)
        let roster = rostersService.getRoster(ownerJID: chat.account, contactJID: chat.participant)
        let account = accountsService.getAccount(jid: chat.account)
        if let client, let roster, let account {
            conversationService = ConversationService(account: account, chat: chat, roster: roster, db: Database()._db, client: client)
            flow = .chat
        }
    }

    func startChat(_ roster: Roster) {
        let client = xmppService.getClient(for: roster.bareJid)
        let account = accountsService.getAccount(jid: roster.bareJid)
        let chat = chatService.getChat(roster.bareJid, roster.contactBareJid)
        if let client, let chat, let account {
            conversationService = ConversationService(account: account, chat: chat, roster: roster, db: Database()._db, client: client)
            flow = .chat
        }
    }
}
