import Combine
import Foundation
import GRDB

final class AccountsService: ObservableObject {
    private weak var database: DatabaseQueue?
    private var cancellables: Set<AnyCancellable> = []

    @Published private(set) var accounts: [Account] = []

    init(_ db: DatabaseQueue) {
        database = db

        doSubscriptions()
    }

    private func doSubscriptions() {
        // watch accounts
        guard let db = database else { return }
        ValueObservation
            .tracking(Account.fetchAll)
            .publisher(in: db, scheduling: .immediate)
            .sink { _ in
                // Handle completion
            } receiveValue: { accounts in
                self.accounts = accounts
            }
            .store(in: &cancellables)
    }

    // MARK: - Account actions
    func getAccount(jid: String) -> Account? {
        accounts.first(where: { $0.bareJid == jid })
    }

    func addAccount(jid: String, password: String) {
        guard let db = database else { return }
        let account = Account(bareJid: jid, pass: password, isActive: true)
        do {
            try db.write { db in
                try account.save(db)
            }
        } catch {
            logIt(.error, "Failed to add account: \(error)")
        }
    }

    func deleteAccount(jid: String) {
        guard let db = database else { return }
        do {
            _ = try db.write { db in
                try Account.deleteOne(db, key: jid)
            }
        } catch {
            logIt(.error, "Failed to delete account: \(error)")
        }
    }
}
