import Combine
import Foundation
import GRDB
import Martin

final class RostersService: ObservableObject {
    private weak var database: DatabaseQueue?
    private weak var xmppService: XMPPService?
    private var cancellables: Set<AnyCancellable> = []

    @Published private(set) var rosters: [Roster] = []

    init(_ db: DatabaseQueue, _ xmpp: XMPPService) {
        database = db
        xmppService = xmpp
        doSubscriptions()
    }

    private func doSubscriptions() {
        // watch rosters
        guard let db = database else { return }
        ValueObservation
            .tracking { db in
                try Roster
                    .filter(Column("locallyDeleted") == false)
                    .fetchAll(db)
            }
            .publisher(in: db, scheduling: .immediate)
            .sink { _ in
                // Handle completion
            } receiveValue: { rosters in
                self.rosters = rosters
            }
            .store(in: &cancellables)
    }

    // MARK: - Roster actions
    func getRoster(ownerJID: String, contactJID: String) -> Roster? {
        rosters.first(where: { $0.bareJid == ownerJID && $0.contactBareJid == contactJID })
    }

    func addRoster(ownerJID: String, contactJID: String, name: String? = nil, groups: [String] = [], completion: @escaping (Result<Void, Error>) -> Void) {
        guard let service = xmppService, let client = service.clients.first(where: { $0.connectionConfiguration.userJid.stringValue == ownerJID }) else {
            return completion(.failure(XMPPError.item_not_found))
        }
        let module = client.modulesManager.module(RosterModule.self)
        module.addItem(jid: JID(contactJID), name: name, groups: groups, completionHandler: { result in
            let result = result
                .map { _ in completion(.success(())) }
                .mapError { $0 as Error }
            completion(result)
        })
    }

    func deleteRoster(ownerJID: String, contactJID: String, completion: @escaping (Result<Void, Error>) -> Void) {
        guard let service = xmppService, let client = service.clients.first(where: { $0.connectionConfiguration.userJid.stringValue == ownerJID }) else {
            return completion(.failure(XMPPError.item_not_found))
        }
        let module = client.modulesManager.module(RosterModule.self)
        module.removeItem(jid: JID(contactJID), completionHandler: { result in
            let result = result
                .map { _ in completion(.success(())) }
                .mapError { $0 as Error }
            completion(result)
        })
    }

    func markRosterAsLocallyDeleted(ownerJID: String, contactJID: String) {
        guard let db = database else { return }
        do {
            _ = try db.write { db in
                try Roster
                    .filter(Column("bareJid") == ownerJID)
                    .filter(Column("contactBareJid") == contactJID)
                    .updateAll(db, Column("locallyDeleted").set(to: true))
            }
        } catch {
            logIt(.error, "Error marking roster as locally deleted: \(error)")
        }
    }

    func editRoster(some _: String) {}

    func rosterByJid(ownerJID: String, contactJID: String) -> Roster? {
        guard let db = database else { return nil }
        do {
            return try db.read { db in
                try Roster
                    .filter(Column("bareJid") == ownerJID)
                    .filter(Column("contactBareJid") == contactJID)
                    .fetchOne(db)
            }
        } catch {
            logIt(.error, "Error fetching roster by JID: \(error)")
            return nil
        }
    }
}
