import SwiftUI

struct ContactsScreen: View {
    @EnvironmentObject var state: AppState

    @State private var addPanelPresented = false
    @State private var isErrorAlertPresented = false
    @State private var errorAlertMessage = ""
    @State private var isShowingLoader = false

    var body: some View {
        ZStack {
            // Background color
            Color.Main.backgroundLight
                .ignoresSafeArea()

            // Content
            VStack(spacing: 0) {
                // Header
                ContactsScreenHeader(addPanelPresented: $addPanelPresented)

                // Contacts list
                if !state.rosters.isEmpty {
                    List {
                        ForEach(state.rosters) { roster in
                            ContactsScreenRow(
                                roster: roster,
                                isErrorAlertPresented: $isErrorAlertPresented,
                                errorAlertMessage: $errorAlertMessage,
                                isShowingLoader: $isShowingLoader
                            )
                        }
                    }
                    .listStyle(.plain)
                    .background(Color.Main.backgroundLight)
                } else {
                    Spacer()
                }

                // Tab bar
                SharedTabBar()
            }
        }
        .loadingIndicator(isShowingLoader)
        .fullScreenCover(isPresented: $addPanelPresented) {
            AddContactOrChannelScreen(isPresented: $addPanelPresented)
        }
        .alert(isPresented: $isErrorAlertPresented) {
            Alert(
                title: Text(L10n.Global.Error.title),
                message: Text(errorAlertMessage),
                dismissButton: .default(Text(L10n.Global.ok))
            )
        }
    }
}

private struct ContactsScreenHeader: View {
    @Binding var addPanelPresented: Bool

    var body: some View {
        ZStack {
            // bg
            Color.Main.backgroundDark
                .ignoresSafeArea()

            // title
            Text(L10n.Contacts.title)
                .font(.head2)
                .foregroundColor(Color.Main.black)

            HStack {
                Spacer()
                Image(systemName: "plus")
                    .foregroundColor(Color.Tango.orangeMedium)
                    .tappablePadding(.symmetric(12)) {
                        addPanelPresented = true
                    }
            }
            .padding(.horizontal, 16)
        }
        .frame(height: 44)
    }
}

private struct ContactsScreenRow: View {
    @EnvironmentObject var state: AppState

    var roster: Roster
    @State private var isShowingMenu = false
    @State private var isDeleteAlertPresented = false

    @Binding var isErrorAlertPresented: Bool
    @Binding var errorAlertMessage: String
    @Binding var isShowingLoader: Bool

    var body: some View {
        VStack(spacing: 0) {
            HStack(spacing: 8) {
                ZStack {
                    Circle()
                        .frame(width: 44, height: 44)
                        .foregroundColor(.red)
                    Text(roster.name?.firstLetter ?? roster.contactBareJid.firstLetter)
                        .foregroundColor(.white)
                        .font(.body1)
                }
                Text(roster.contactBareJid)
                    .foregroundColor(Color.Main.black)
                    .font(.body2)
                Spacer()
            }
            .padding(.horizontal, 16)
            .padding(.vertical, 4)
            Rectangle()
                .frame(maxWidth: .infinity)
                .frame(height: 1)
                .foregroundColor(.Main.backgroundDark)
        }
        .sharedListRow()
        .onTapGesture {
            state.startChat(roster)
        }
        .onLongPressGesture {
            isShowingMenu.toggle()
        }
        .contextMenu {
            Button(L10n.Contacts.sendMessage, systemImage: "message") {
                state.startChat(roster)
            }
            Divider()

            Button(L10n.Contacts.editContact) {
                print("Edit contact")
            }

            Button(L10n.Contacts.selectContact) {
                print("Select contact")
            }

            Divider()
            Button(L10n.Contacts.deleteContact, systemImage: "trash", role: .destructive) {
                isDeleteAlertPresented = true
            }
        }
        .alert(isPresented: $isDeleteAlertPresented) {
            Alert(
                title: Text(L10n.Contacts.Delete.title),
                message: Text(L10n.Contacts.Delete.message),
                // mark locally as deleted
                primaryButton: .destructive(Text(L10n.Contacts.Delete.deleteFromDevice)) {
                    state.markRosterAsLocallyDeleted(ownerJID: roster.bareJid, contactJID: roster.contactBareJid)
                },
                // delete from server
                secondaryButton: .destructive(Text(L10n.Contacts.Delete.deleteCompletely)) {
                    isShowingLoader = true
                    state.deleteRoster(ownerJID: roster.bareJid, contactJID: roster.contactBareJid) { result in
                        isShowingLoader = false
                        if case .failure = result {
                            isErrorAlertPresented = true
                            errorAlertMessage = L10n.Contacts.Delete.error
                        }
                    }
                }
            )
        }
    }
}
