import Combine
import Martin
import SwiftUI

struct LoginScreen: View {
    @EnvironmentObject var state: AppState

    enum Field {
        case userJid
        case password
    }

    @FocusState private var focus: Field?
    @State private var errorMsg: String = ""
    @State private var isShowingAlert = false
    @State private var isShowingLoader = false

    #if DEBUG
        @State private var jidStr: String = "test1@test.uwu.do"
        @State private var pass: String = "12345"
    #else
        @State private var jidStr: String = ""
        @State private var pass: String = ""
    #endif

    public var body: some View {
        ZStack {
            // background
            Color.Main.backgroundLight
                .ignoresSafeArea()

            // content
            VStack(spacing: 32) {
                // icon
                Image.logo
                    .resizable()
                    .aspectRatio(contentMode: .fit)
                    .frame(width: 120, height: 120)

                // texts
                VStack(spacing: 10) {
                    Text(L10n.Login.title)
                        .font(.head1l)
                        .foregroundColor(.Tango.blueMedium)
                        .fixedSize(horizontal: true, vertical: false)
                    Text(L10n.Login.subtitle)
                        .font(.body2)
                        .foregroundColor(.Tango.blueLight)
                        .multilineTextAlignment(.center)
                        .fixedSize(horizontal: false, vertical: true)
                }

                VStack(spacing: 16) {
                    UniversalInputCollection.TextField(
                        prompt: L10n.Login.Hint.jid,
                        text: $jidStr,
                        focus: $focus,
                        fieldType: .userJid,
                        contentType: .emailAddress,
                        keyboardType: .emailAddress,
                        submitLabel: .next,
                        action: {
                            focus = .password
                        }
                    )
                    UniversalInputCollection.SecureField(
                        prompt: L10n.Login.Hint.password,
                        text: $pass,
                        focus: $focus,
                        fieldType: .password,
                        submitLabel: .go,
                        action: {
                            focus = nil
                        }
                    )

                    Button {
                        isShowingLoader = true
                        state.addAccount(jid: jidStr, password: pass)
                    } label: {
                        Text(L10n.Login.btn)
                    }
                    .buttonStyle(PrimaryButtonStyle())
                    .disabled(!loginInputValid)

                    Button {
                        state.flow = .welcome
                    } label: {
                        Text("\(Image(systemName: "chevron.left")) \(L10n.Global.back)")
                            .foregroundColor(.Tango.blueLight)
                            .font(.body2)
                    }
                }
            }
            .padding(.horizontal, 32)
        }
        .loadingIndicator(isShowingLoader)
        .alert(isPresented: $isShowingAlert) {
            Alert(
                title: Text(L10n.Global.Error.title),
                message: Text(errorMsg),
                dismissButton: .default(Text(L10n.Global.ok))
            )
        }
        .onReceive(clientStatePublisher) { clientState in
            switch clientState {
            case .connected:
                isShowingLoader = false
                state.flow = .chatsList

            case .disconnected(let reason):
                state.deleteAccount(jid: jidStr)
                isShowingLoader = false
                switch reason {
                case .authenticationFailure:
                    isShowingAlert = true
                    errorMsg = L10n.Login.Error.wrongPassword

                case .noRouteToServer:
                    isShowingAlert = true
                    errorMsg = L10n.Login.Error.noServer

                default:
                    isShowingAlert = true
                    errorMsg = L10n.Login.Error.serverError
                }

            default:
                break
            }
        }
    }

    private var loginInputValid: Bool {
        !jidStr.isEmpty && !pass.isEmpty && UniversalInputCollection.Validators.isEmail(jidStr)
    }

    private var clientStatePublisher: AnyPublisher<XMPPClient.State?, Never> {
        if let client = state.clients.first(where: { $0.userBareJid.stringValue == jidStr }) {
            return client.$state
                .dropFirst()
                .receive(on: DispatchQueue.main)
                .map { $0 }
                .eraseToAnyPublisher()
        } else {
            return Just(nil)
                .setFailureType(to: Never.self)
                .eraseToAnyPublisher()
        }
    }
}
